import xml.etree.ElementTree as ET
from typing import List, Dict, Any


class DrawioGenerator:
    """draw.io (mxGraph) XML生成ユーティリティ - テーブル形式"""

    @staticmethod
    def create_mxgraph_model(cells: List[Dict[str, Any]], width: int = 1200, height: int = 800) -> str:
        """
        mxGraphModelのXML文字列を生成

        Args:
            cells: mxCellのデータリスト
            width: キャンバス幅
            height: キャンバス高さ

        Returns:
            XML文字列（非圧縮形式）
        """
        root = ET.Element('mxGraphModel')
        root.set('dx', '1106')
        root.set('dy', '776')
        root.set('grid', '1')
        root.set('gridSize', '10')
        root.set('guides', '1')
        root.set('tooltips', '1')
        root.set('connect', '1')
        root.set('arrows', '1')
        root.set('fold', '1')
        root.set('page', '1')
        root.set('pageScale', '1')
        root.set('pageWidth', str(width))
        root.set('pageHeight', str(height))
        root.set('math', '0')
        root.set('shadow', '0')

        root_elem = ET.SubElement(root, 'root')

        # デフォルトセル（id=0, 1）
        cell0 = ET.SubElement(root_elem, 'mxCell')
        cell0.set('id', '0')

        cell1 = ET.SubElement(root_elem, 'mxCell')
        cell1.set('id', '1')
        cell1.set('parent', '0')

        # 各セルを追加
        for cell_data in cells:
            DrawioGenerator._create_cell_element(root_elem, cell_data)

        # XML文字列に変換（整形付き）
        ET.indent(root, space='  ', level=0)
        return ET.tostring(root, encoding='unicode')

    @staticmethod
    def _create_cell_element(parent: ET.Element, cell_data: Dict[str, Any]):
        """
        mxCell要素を生成

        Args:
            parent: 親XML要素
            cell_data: セルのデータ辞書
        """
        cell = ET.SubElement(parent, 'mxCell')

        for key, value in cell_data.items():
            if key == 'geometry':
                # mxGeometry要素を作成
                geom = ET.SubElement(cell, 'mxGeometry')
                for g_key, g_value in value.items():
                    geom.set(g_key, str(g_value))
            else:
                # 属性として設定
                cell.set(key, str(value))

    @staticmethod
    def create_group_cell(cell_id: str, x: int, y: int, width: int, height: int,
                          parent: str = '1') -> Dict[str, Any]:
        """
        グループセルを生成（複数要素をまとめる親コンテナ）

        Args:
            cell_id: セルID
            x, y: 位置
            width, height: サイズ
            parent: 親セルID

        Returns:
            mxCellデータ辞書
        """
        style = 'group;'

        return {
            'id': cell_id,
            'value': '',
            'style': style,
            'vertex': '1',
            'connectable': '0',
            'parent': parent,
            'geometry': {
                'x': x,
                'y': y,
                'width': width,
                'height': height,
                'as': 'geometry'
            }
        }

    @staticmethod
    def create_table_rect(cell_id: str, x: int, y: int, width: int, height: int,
                          bgcolor: str = '#ffffff', use_gradient: bool = False,
                          parent: str = '1') -> Dict[str, Any]:
        """
        テーブル外枠の矩形を生成

        Args:
            cell_id: セルID
            x, y: 位置
            width, height: サイズ
            bgcolor: 背景色
            use_gradient: グラデーション使用フラグ
            parent: 親セルID

        Returns:
            mxCellデータ辞書
        """
        style = f'rounded=0;whiteSpace=wrap;html=1;fillColor={bgcolor};strokeColor=#000000;strokeWidth=2;'
        if use_gradient:
            style += 'gradientColor=#ffffff;gradientDirection=east;'

        return {
            'id': cell_id,
            'value': '',
            'style': style,
            'vertex': '1',
            'parent': parent,
            'geometry': {
                'x': x,
                'y': y,
                'width': width,
                'height': height,
                'as': 'geometry'
            }
        }

    @staticmethod
    def create_header_rect(cell_id: str, x: int, y: int, width: int, height: int,
                           bgcolor: str = '#ffffff', use_gradient: bool = False,
                           parent: str = '1') -> Dict[str, Any]:
        """
        ヘッダー背景の矩形を生成

        Args:
            cell_id: セルID
            x, y: 位置
            width, height: サイズ
            bgcolor: 背景色
            use_gradient: グラデーション使用フラグ
            parent: 親セルID

        Returns:
            mxCellデータ辞書
        """
        style = f'rounded=0;whiteSpace=wrap;html=1;fillColor={bgcolor};strokeColor=#000000;strokeWidth=1;'
        if use_gradient:
            style += 'gradientColor=#ffffff;gradientDirection=east;'

        return {
            'id': cell_id,
            'value': '',
            'style': style,
            'vertex': '1',
            'parent': parent,
            'geometry': {
                'x': x,
                'y': y,
                'width': width,
                'height': height,
                'as': 'geometry'
            }
        }

    @staticmethod
    def create_text_cell(cell_id: str, value: str, x: int, y: int, width: int, height: int,
                         font_size: int = 10, font_weight: str = 'normal',
                         align: str = 'left', parent: str = '1') -> Dict[str, Any]:
        """
        テキストセルを生成

        Args:
            cell_id: セルID
            value: 表示テキスト
            x, y: 位置
            width, height: サイズ
            font_size: フォントサイズ
            font_weight: フォントウェイト (normal/bold)
            align: テキスト配置 (left/center/right)
            parent: 親セルID

        Returns:
            mxCellデータ辞書
        """
        font_style = '1' if font_weight == 'bold' else '0'
        style = (
            f'text;html=1;strokeColor=none;fillColor=none;align={align};'
            f'verticalAlign=middle;whiteSpace=wrap;rounded=0;'
            f'fontSize={font_size};fontStyle={font_style};'
        )

        return {
            'id': cell_id,
            'value': value,
            'style': style,
            'vertex': '1',
            'parent': parent,
            'geometry': {
                'x': x,
                'y': y,
                'width': width,
                'height': height,
                'as': 'geometry'
            }
        }

    @staticmethod
    def create_marker_rect(cell_id: str, x: int, y: int, width: int, height: int,
                           parent: str = '1') -> Dict[str, Any]:
        """
        NOT NULLマーカー（黒四角）を生成

        Args:
            cell_id: セルID
            x, y: 位置
            width, height: サイズ
            parent: 親セルID

        Returns:
            mxCellデータ辞書
        """
        style = 'rounded=0;whiteSpace=wrap;html=1;fillColor=#000000;strokeColor=none;'

        return {
            'id': cell_id,
            'value': '',
            'style': style,
            'vertex': '1',
            'parent': parent,
            'geometry': {
                'x': x,
                'y': y,
                'width': width,
                'height': height,
                'as': 'geometry'
            }
        }

    @staticmethod
    def create_line(cell_id: str, x1: int, y1: int, x2: int, y2: int,
                    stroke_width: int = 1, parent: str = '1') -> Dict[str, Any]:
        """
        線を生成

        Args:
            cell_id: セルID
            x1, y1: 開始位置
            x2, y2: 終了位置
            stroke_width: 線の太さ
            parent: 親セルID

        Returns:
            mxCellデータ辞書
        """
        # 線の長さと位置を計算
        width = abs(x2 - x1) if x2 != x1 else 1
        height = abs(y2 - y1) if y2 != y1 else 1
        x = min(x1, x2)
        y = min(y1, y2)

        if x2 == x1:
            # 縦線
            style = f'endArrow=none;html=1;strokeWidth={stroke_width};'
        else:
            # 横線
            style = f'endArrow=none;html=1;strokeWidth={stroke_width};'

        return {
            'id': cell_id,
            'value': '',
            'style': style,
            'edge': '1',
            'parent': parent,
            'geometry': {
                'x': x,
                'y': y,
                'width': width,
                'height': height,
                'relative': '1',
                'as': 'geometry'
            }
        }

    @staticmethod
    def create_horizontal_line(cell_id: str, x: int, y: int, width: int,
                               stroke_width: int = 1, parent: str = '1') -> Dict[str, Any]:
        """
        水平線を生成（shape=lineを使用）

        Args:
            cell_id: セルID
            x, y: 開始位置
            width: 線の長さ
            stroke_width: 線の太さ
            parent: 親セルID

        Returns:
            mxCellデータ辞書
        """
        style = f'line;strokeWidth={stroke_width};fillColor=none;align=left;verticalAlign=middle;spacingTop=-1;spacingLeft=3;spacingRight=3;rotatable=0;labelPosition=right;points=[];portConstraint=eastwest;strokeColor=#000000;'

        return {
            'id': cell_id,
            'value': '',
            'style': style,
            'vertex': '1',
            'connectable': '0',
            'parent': parent,
            'geometry': {
                'x': x,
                'y': y,
                'width': width,
                'height': 1,
                'as': 'geometry'
            }
        }

    @staticmethod
    def create_edge_cell(cell_id: str, from_cell_id: str, to_cell_id: str, style: str, parent: str = '1') -> Dict[str, Any]:
        """
        エッジ（関係線）用のmxCellデータを生成

        Args:
            cell_id: セルID
            from_cell_id: 開始ノードのセルID
            to_cell_id: 終了ノードのセルID
            style: エッジのスタイル文字列
            parent: 親セルID

        Returns:
            mxCellデータ辞書
        """
        return {
            'id': cell_id,
            'value': '',
            'style': style,
            'edge': '1',
            'parent': parent,
            'source': from_cell_id,
            'target': to_cell_id,
            'geometry': {
                'relative': '1',
                'as': 'geometry'
            }
        }



from dataclasses import dataclass
from ...core.models import LineType, Cardinality


@dataclass
class Edge:
    from_node_id: str
    to_node_id: str
    line_type: LineType = LineType.STRAIGHT
    cardinality: Cardinality = None

    def __post_init__(self):
        if self.cardinality is None:
            self.cardinality = Cardinality()

    def render(self, from_x: int, from_y: int, to_x: int, to_y: int, line_type: LineType, from_edge: str = None, to_edge: str = None) -> str:
        def get_perpendicular_point(x, y, edge_type, length):
            """接地面（外枠の辺）に対して垂直に伸ばした点を計算"""
            if edge_type == 'left':
                return x - length, y
            elif edge_type == 'right':
                return x + length, y
            elif edge_type == 'top':
                return x, y - length
            elif edge_type == 'bottom':
                return x, y + length
            else:
                return x, y

        def render_ie_notation_symbol(x: int, y: int, cardinality: str, edge_type: str) -> tuple:
            """IE記法でカーディナリティ記号を描画し、新しい接続点を返す"""
            symbols = []
            new_x, new_y = x, y

            if cardinality == "0":
                if edge_type == 'left':
                    circle_x = x - 8
                    symbols.append(f'<circle cx="{circle_x}" cy="{y}" r="3" fill="none" stroke="black" stroke-width="1"/>')
                    new_x = circle_x - 3
                elif edge_type == 'right':
                    circle_x = x + 8
                    symbols.append(f'<circle cx="{circle_x}" cy="{y}" r="3" fill="none" stroke="black" stroke-width="1"/>')
                    new_x = circle_x + 3
                elif edge_type == 'top':
                    circle_y = y - 8
                    symbols.append(f'<circle cx="{x}" cy="{circle_y}" r="3" fill="none" stroke="black" stroke-width="1"/>')
                    new_y = circle_y - 3
                elif edge_type == 'bottom':
                    circle_y = y + 8
                    symbols.append(f'<circle cx="{x}" cy="{circle_y}" r="3" fill="none" stroke="black" stroke-width="1"/>')
                    new_y = circle_y + 3

            elif not cardinality or cardinality == "1":
                if edge_type == 'left':
                    cross_x = x - 8
                    symbols.append(f'<line x1="{cross_x}" y1="{y-4}" x2="{cross_x}" y2="{y+4}" stroke="black" stroke-width="2"/>')
                    symbols.append(f'<line x1="{cross_x-4}" y1="{y}" x2="{cross_x+4}" y2="{y}" stroke="black" stroke-width="2"/>')
                    new_x = cross_x - 4
                elif edge_type == 'right':
                    cross_x = x + 8
                    symbols.append(f'<line x1="{cross_x}" y1="{y-4}" x2="{cross_x}" y2="{y+4}" stroke="black" stroke-width="2"/>')
                    symbols.append(f'<line x1="{cross_x-4}" y1="{y}" x2="{cross_x+4}" y2="{y}" stroke="black" stroke-width="2"/>')
                    new_x = cross_x + 4
                elif edge_type == 'top':
                    cross_y = y - 8
                    symbols.append(f'<line x1="{x}" y1="{cross_y-4}" x2="{x}" y2="{cross_y+4}" stroke="black" stroke-width="2"/>')
                    symbols.append(f'<line x1="{x-4}" y1="{cross_y}" x2="{x+4}" y2="{cross_y}" stroke="black" stroke-width="2"/>')
                    new_y = cross_y - 4
                elif edge_type == 'bottom':
                    cross_y = y + 8
                    symbols.append(f'<line x1="{x}" y1="{cross_y-4}" x2="{x}" y2="{cross_y+4}" stroke="black" stroke-width="2"/>')
                    symbols.append(f'<line x1="{x-4}" y1="{cross_y}" x2="{x+4}" y2="{cross_y}" stroke="black" stroke-width="2"/>')
                    new_y = cross_y + 4

            elif cardinality == "0..1":
                if edge_type == 'left':
                    circle_x = x - 8
                    line_x = x - 15
                    symbols.append(f'<circle cx="{circle_x}" cy="{y}" r="3" fill="none" stroke="black" stroke-width="1"/>')
                    symbols.append(f'<line x1="{line_x}" y1="{y-4}" x2="{line_x}" y2="{y+4}" stroke="black" stroke-width="2"/>')
                    new_x = line_x
                elif edge_type == 'right':
                    circle_x = x + 8
                    line_x = x + 15
                    symbols.append(f'<circle cx="{circle_x}" cy="{y}" r="3" fill="none" stroke="black" stroke-width="1"/>')
                    symbols.append(f'<line x1="{line_x}" y1="{y-4}" x2="{line_x}" y2="{y+4}" stroke="black" stroke-width="2"/>')
                    new_x = line_x
                elif edge_type == 'top':
                    circle_y = y - 8
                    line_y = y - 15
                    symbols.append(f'<circle cx="{x}" cy="{circle_y}" r="3" fill="none" stroke="black" stroke-width="1"/>')
                    symbols.append(f'<line x1="{x-4}" y1="{line_y}" x2="{x+4}" y2="{line_y}" stroke="black" stroke-width="2"/>')
                    new_y = line_y
                elif edge_type == 'bottom':
                    circle_y = y + 8
                    line_y = y + 15
                    symbols.append(f'<circle cx="{x}" cy="{circle_y}" r="3" fill="none" stroke="black" stroke-width="1"/>')
                    symbols.append(f'<line x1="{x-4}" y1="{line_y}" x2="{x+4}" y2="{line_y}" stroke="black" stroke-width="2"/>')
                    new_y = line_y

            elif cardinality in ["*", "1..*", "0..*", "0...*", "1...*", "many"]:
                if edge_type == 'left':
                    crow_x = x - 12
                    symbols.append(f'<line x1="{crow_x}" y1="{y}" x2="{crow_x+8}" y2="{y-5}" stroke="black" stroke-width="1"/>')
                    symbols.append(f'<line x1="{crow_x}" y1="{y}" x2="{crow_x+8}" y2="{y+5}" stroke="black" stroke-width="1"/>')
                    symbols.append(f'<line x1="{crow_x}" y1="{y}" x2="{crow_x+8}" y2="{y}" stroke="black" stroke-width="1"/>')
                    new_x = crow_x
                elif edge_type == 'right':
                    crow_x = x + 12
                    symbols.append(f'<line x1="{crow_x}" y1="{y}" x2="{crow_x-8}" y2="{y-5}" stroke="black" stroke-width="1"/>')
                    symbols.append(f'<line x1="{crow_x}" y1="{y}" x2="{crow_x-8}" y2="{y+5}" stroke="black" stroke-width="1"/>')
                    symbols.append(f'<line x1="{crow_x}" y1="{y}" x2="{crow_x-8}" y2="{y}" stroke="black" stroke-width="1"/>')
                    new_x = crow_x
                elif edge_type == 'top':
                    crow_y = y - 12
                    symbols.append(f'<line x1="{x}" y1="{crow_y}" x2="{x-5}" y2="{crow_y+8}" stroke="black" stroke-width="1"/>')
                    symbols.append(f'<line x1="{x}" y1="{crow_y}" x2="{x+5}" y2="{crow_y+8}" stroke="black" stroke-width="1"/>')
                    symbols.append(f'<line x1="{x}" y1="{crow_y}" x2="{x}" y2="{crow_y+8}" stroke="black" stroke-width="1"/>')
                    new_y = crow_y
                elif edge_type == 'bottom':
                    crow_y = y + 12
                    symbols.append(f'<line x1="{x}" y1="{crow_y}" x2="{x-5}" y2="{crow_y-8}" stroke="black" stroke-width="1"/>')
                    symbols.append(f'<line x1="{x}" y1="{crow_y}" x2="{x+5}" y2="{crow_y-8}" stroke="black" stroke-width="1"/>')
                    symbols.append(f'<line x1="{x}" y1="{crow_y}" x2="{x}" y2="{crow_y-8}" stroke="black" stroke-width="1"/>')
                    new_y = crow_y

            return '\n'.join(symbols), new_x, new_y

        # IE記号の位置計算と描画
        from_symbols = ""
        to_symbols = ""
        final_from_x, final_from_y = from_x, from_y
        final_to_x, final_to_y = to_x, to_y

        if self.cardinality:
            if self.cardinality.from_side:
                from_symbols, final_from_x, final_from_y = render_ie_notation_symbol(from_x, from_y, self.cardinality.from_side, from_edge)

            if self.cardinality.to_side:
                to_symbols, final_to_x, final_to_y = render_ie_notation_symbol(to_x, to_y, self.cardinality.to_side, to_edge)

        svg_parts = []

        if line_type == LineType.STRAIGHT:
            svg_parts.append(f'<line x1="{final_from_x}" y1="{final_from_y}" x2="{final_to_x}" y2="{final_to_y}" stroke="black" stroke-width="1"/>')

        elif line_type == LineType.CRANK:
            from_perp_x, from_perp_y = get_perpendicular_point(final_from_x, final_from_y, from_edge, 30)
            to_perp_x, to_perp_y = get_perpendicular_point(final_to_x, final_to_y, to_edge, 30)

            if from_edge in ['top', 'bottom'] and to_edge in ['top', 'bottom']:
                mid1_x, mid1_y = from_perp_x, from_perp_y
                mid2_x, mid2_y = final_to_x, from_perp_y
            elif from_edge in ['left', 'right'] and to_edge in ['left', 'right']:
                mid1_x, mid1_y = from_perp_x, from_perp_y
                mid2_x, mid2_y = from_perp_x, final_to_y
            elif from_edge in ['top', 'bottom'] and to_edge in ['left', 'right']:
                mid1_x, mid1_y = from_perp_x, from_perp_y
                mid2_x, mid2_y = to_perp_x, from_perp_y
            elif from_edge in ['left', 'right'] and to_edge in ['top', 'bottom']:
                mid1_x, mid1_y = from_perp_x, from_perp_y
                mid2_x, mid2_y = from_perp_x, to_perp_y
            else:
                mid1_x, mid1_y = from_perp_x, from_perp_y
                mid2_x, mid2_y = to_perp_x, to_perp_y

            if from_edge in ['top', 'bottom'] and to_edge in ['top', 'bottom']:
                svg_parts.append(f'<path d="M {final_from_x} {final_from_y} L {from_perp_x} {from_perp_y} L {mid2_x} {mid2_y} L {final_to_x} {final_to_y}" stroke="black" stroke-width="1" fill="none"/>')
            elif from_edge in ['left', 'right'] and to_edge in ['left', 'right']:
                svg_parts.append(f'<path d="M {final_from_x} {final_from_y} L {from_perp_x} {from_perp_y} L {mid2_x} {mid2_y} L {final_to_x} {final_to_y}" stroke="black" stroke-width="1" fill="none"/>')
            else:
                svg_parts.append(f'<path d="M {final_from_x} {final_from_y} L {from_perp_x} {from_perp_y} L {mid1_x} {mid1_y} L {mid2_x} {mid2_y} L {to_perp_x} {to_perp_y} L {final_to_x} {final_to_y}" stroke="black" stroke-width="1" fill="none"/>')

        elif line_type == LineType.SPLINE:
            from_perp_x, from_perp_y = get_perpendicular_point(final_from_x, final_from_y, from_edge, 50)
            to_perp_x, to_perp_y = get_perpendicular_point(final_to_x, final_to_y, to_edge, 50)

            svg_parts.append(f'<path d="M {final_from_x} {final_from_y} C {from_perp_x} {from_perp_y} {to_perp_x} {to_perp_y} {final_to_x} {final_to_y}" stroke="black" stroke-width="1" fill="none"/>')

        else:
            svg_parts.append(f'<line x1="{final_from_x}" y1="{final_from_y}" x2="{final_to_x}" y2="{final_to_y}" stroke="black" stroke-width="1"/>')

        if from_symbols:
            svg_parts.append(from_symbols)
        if to_symbols:
            svg_parts.append(to_symbols)

        return '\n'.join(svg_parts)



from typing import List, Dict, Any
from ...core.models import LineType


class DrawioCanvas:
    """draw.io用のキャンバス管理クラス"""

    def __init__(
        self,
        default_line_type: LineType = LineType.STRAIGHT,
        width: int = 1200,
        height: int = 800
    ):
        self.width = width
        self.height = height
        self.default_line_type = default_line_type
        self.nodes: List['DrawioNode'] = []
        self.edges: List = []
        self.next_position = [50, 50]
        self.column_width = 450
        self.row_height = 200
        self.next_cell_id = 2

    def get_next_cell_id(self) -> str:
        """次のmxCell IDを取得"""
        cell_id = str(self.next_cell_id)
        self.next_cell_id += 1
        return cell_id

    def add_node(self, node: 'DrawioNode'):
        """ノードを追加"""
        self.nodes.append(node)

    def get_node(self, node_id: str) -> 'DrawioNode':
        """ノードIDからノードを取得"""
        for node in self.nodes:
            if node.node_id == node_id:
                return node
        return None


class DrawioNode:
    """draw.io用のノード（テーブル）を表すクラス"""

    def __init__(self, node_id: str, stencil, data: Dict[str, Any], x: int, y: int, cell_id: str):
        self.node_id = node_id
        self.cell_id = cell_id
        self.stencil = stencil
        self.data = data
        self.x = x
        self.y = y
        self.width = self._calculate_width()
        self.height = self._calculate_height()

    def _calculate_height(self) -> int:
        """高さを計算"""
        if hasattr(self.stencil, 'calculate_height'):
            return self.stencil.calculate_height(self.data)
        return 100

    def _calculate_width(self) -> int:
        """幅を計算"""
        if hasattr(self.stencil, 'get_width'):
            return self.stencil.get_width(self.data)
        return 200



"""共通コアモジュール"""
from .models import LineType, Cardinality, Column, Table
from .text_metrics import calculate_text_width
from .layout import LayoutEngine

__all__ = [
    'LineType',
    'Cardinality',
    'Column',
    'Table',
    'calculate_text_width',
    'LayoutEngine',
]



"""
in4viz - ER図可視化ライブラリ

SVGとdraw.io形式での出力をサポートする統合ER図作成ライブラリ
"""

from .core.models import Table, Column, LineType, Cardinality
from .backends.svg import SVGERDiagram
from .backends.drawio import DrawioERDiagram

__version__ = '0.5.0'

__all__ = [
    'Table',
    'Column',
    'LineType',
    'Cardinality',
    'SVGERDiagram',
    'DrawioERDiagram',
]



from ...core.models import LineType, Cardinality


class DrawioEdge:
    """draw.io用のエッジ（関係線）クラス"""

    def __init__(self, from_node_id: str, to_node_id: str, line_type: LineType = LineType.STRAIGHT, cardinality: Cardinality = None):
        self.from_node_id = from_node_id
        self.to_node_id = to_node_id
        self.line_type = line_type
        self.cardinality = cardinality if cardinality is not None else Cardinality()

    def _map_cardinality_to_arrow(self, cardinality_str: str) -> str:
        """
        カーディナリティ文字列をdraw.ioの矢印スタイルにマッピング

        Args:
            cardinality_str: カーディナリティ文字列（"1", "0..1", "*", etc.）

        Returns:
            draw.io矢印スタイル（ERone, ERmany, ERzeroToOne, etc.）
        """
        if cardinality_str in ["*", "1..*", "0..*", "0...*", "1...*", "many"]:
            return "ERmany"
        elif cardinality_str == "0..1":
            return "ERzeroToOne"
        elif cardinality_str == "0":
            return "ERzeroToOne"
        elif cardinality_str == "1" or not cardinality_str:
            return "ERone"
        else:
            return "ERone"

    def _build_edge_style(self) -> str:
        """
        エッジのスタイル文字列を構築

        Returns:
            draw.io用のスタイル文字列
        """
        style_parts = []

        # LineTypeに応じたedgeStyle
        if self.line_type == LineType.CRANK:
            style_parts.append("edgeStyle=orthogonalEdgeStyle")
            style_parts.append("rounded=0")
            style_parts.append("orthogonalLoop=1")
            style_parts.append("jettySize=auto")
        elif self.line_type == LineType.SPLINE:
            style_parts.append("edgeStyle=elbowEdgeStyle")
            style_parts.append("curved=1")
            style_parts.append("rounded=1")
        # STRAIGHTの場合はedgeStyleを省略

        # 基本スタイル
        style_parts.append("html=1")

        # カーディナリティから矢印スタイルを決定
        if self.cardinality:
            if self.cardinality.from_side:
                start_arrow = self._map_cardinality_to_arrow(self.cardinality.from_side)
                style_parts.append(f"startArrow={start_arrow}")

            if self.cardinality.to_side:
                end_arrow = self._map_cardinality_to_arrow(self.cardinality.to_side)
                style_parts.append(f"endArrow={end_arrow}")

        return ";".join(style_parts) + ";"

    def render_mxcell_data(self, from_cell_id: str, to_cell_id: str, edge_cell_id: str) -> dict:
        """
        エッジのmxCellデータを生成

        Args:
            from_cell_id: 開始ノードのmxCell ID
            to_cell_id: 終了ノードのmxCell ID
            edge_cell_id: エッジ自身のmxCell ID

        Returns:
            mxCellデータ辞書
        """
        from .generator import DrawioGenerator

        style = self._build_edge_style()
        return DrawioGenerator.create_edge_cell(
            cell_id=edge_cell_id,
            from_cell_id=from_cell_id,
            to_cell_id=to_cell_id,
            style=style
        )



from typing import List
from enum import Enum
from dataclasses import dataclass


class LineType(Enum):
    STRAIGHT = "straight"
    CRANK = "crank"
    SPLINE = "spline"


@dataclass
class Cardinality:
    from_side: str = "1"  # "1", "0..1", "1..*", "0..*"
    to_side: str = "1"    # "1", "0..1", "1..*", "0..*"


@dataclass
class Column:
    name: str
    logical_name: str
    type: str
    primary_key: bool = False
    nullable: bool = True
    foreign_key: bool = False
    index: bool = False


@dataclass
class Table:
    name: str
    logical_name: str
    columns: List[Column]
    bgcolor: str = '#ffffff'
    use_gradient: bool = False



"""文字幅計算のための共通ユーティリティ"""


def calculate_text_width(text: str, is_japanese: bool = True) -> int:
    """
    テキストの幅を計算（日本語/英語を区別）

    Args:
        text: 計算対象のテキスト
        is_japanese: True なら日本語として計算、False なら英語として計算

    Returns:
        計算されたテキスト幅（ピクセル）
    """
    if is_japanese:
        return len(text) * 11
    else:
        return len(text) * 6



from typing import List, Dict, Any, Tuple
from ...core.models import LineType
from .rendering import Edge
from .stencil import Stencil


class Canvas:
    def __init__(self, width: int = 1200, height: int = 800, default_line_type: LineType = LineType.STRAIGHT):
        self.width = width
        self.height = height
        self.default_line_type = default_line_type
        self.nodes: List['Node'] = []
        self.edges: List[Edge] = []
        self.next_position = [50, 50]
        self.column_width = 450
        self.row_height = 200

    def add_node(self, node: 'Node'):
        self.nodes.append(node)

    def add_edge(self, from_node_id: str, to_node_id: str, line_type: LineType = None):
        if line_type is None:
            line_type = self.default_line_type
        edge = Edge(from_node_id, to_node_id, line_type)
        self.edges.append(edge)

    def get_node(self, node_id: str) -> 'Node':
        for node in self.nodes:
            if node.node_id == node_id:
                return node
        return None

    def _get_node_edge_points(self, from_node: 'Node', to_node: 'Node') -> Tuple[int, int, int, int, str, str]:
        """ステンシル中央に向かって線を引き、外枠で止めるポイントを計算"""
        def get_line_rect_intersection(center_x, center_y, target_x, target_y, rect_x, rect_y, rect_w, rect_h):
            """中央から目標点への線が外枠と交わる最適な点を計算"""
            dx = target_x - center_x
            dy = target_y - center_y

            if dx == 0 and dy == 0:
                return center_x, center_y, 'center'

            # 主方向を判定して適切な辺を選択
            if abs(dx) > abs(dy):
                # 水平方向が主
                if dx > 0:
                    # 右向き → 右辺から出る
                    x = rect_x + rect_w
                    t = (x - center_x) / dx
                    y = center_y + t * dy
                    if rect_y <= y <= rect_y + rect_h:
                        return x, y, 'right'
                else:
                    # 左向き → 左辺から出る
                    x = rect_x
                    t = (x - center_x) / dx
                    y = center_y + t * dy
                    if rect_y <= y <= rect_y + rect_h:
                        return x, y, 'left'
            else:
                # 垂直方向が主
                if dy > 0:
                    # 下向き → 下辺から出る
                    y = rect_y + rect_h
                    t = (y - center_y) / dy
                    x = center_x + t * dx
                    if rect_x <= x <= rect_x + rect_w:
                        return x, y, 'bottom'
                else:
                    # 上向き → 上辺から出る
                    y = rect_y
                    t = (y - center_y) / dy
                    x = center_x + t * dx
                    if rect_x <= x <= rect_x + rect_w:
                        return x, y, 'top'

            # フォールバック：最近傍辺を強制選択
            edges = [
                (rect_x, center_y, 'left'),
                (rect_x + rect_w, center_y, 'right'),
                (center_x, rect_y, 'top'),
                (center_x, rect_y + rect_h, 'bottom')
            ]

            best_edge = None
            min_dist = float('inf')
            for edge_x, edge_y, edge_name in edges:
                clipped_x = max(rect_x, min(rect_x + rect_w, edge_x))
                clipped_y = max(rect_y, min(rect_y + rect_h, edge_y))
                dist = abs(target_x - clipped_x) + abs(target_y - clipped_y)
                if dist < min_dist:
                    min_dist = dist
                    best_edge = (clipped_x, clipped_y, edge_name)

            return best_edge[0], best_edge[1], best_edge[2]

        from_center_x = from_node.x + from_node.width // 2
        from_center_y = from_node.y + from_node.height // 2
        to_center_x = to_node.x + to_node.width // 2
        to_center_y = to_node.y + to_node.height // 2

        # fromノードの外枠交点
        from_x, from_y, from_edge = get_line_rect_intersection(
            from_center_x, from_center_y, to_center_x, to_center_y,
            from_node.x, from_node.y, from_node.width, from_node.height
        )

        # toノードの外枠交点
        to_x, to_y, to_edge = get_line_rect_intersection(
            to_center_x, to_center_y, from_center_x, from_center_y,
            to_node.x, to_node.y, to_node.width, to_node.height
        )

        return int(from_x), int(from_y), int(to_x), int(to_y), from_edge, to_edge

    def render_edges(self) -> List[str]:
        edge_parts = []
        for edge in self.edges:
            from_node = self.get_node(edge.from_node_id)
            to_node = self.get_node(edge.to_node_id)

            if from_node and to_node:
                from_x, from_y, to_x, to_y, from_edge, to_edge = self._get_node_edge_points(from_node, to_node)
                edge_parts.append(edge.render(from_x, from_y, to_x, to_y, edge.line_type, from_edge, to_edge))

        return edge_parts

    def render_arrow_marker(self) -> str:
        return '''<defs>
            <marker id="arrowhead" markerWidth="10" markerHeight="7"
                    refX="10" refY="3.5" orient="auto">
                <polygon points="0 0, 10 3.5, 0 7" fill="black"/>
            </marker>
        </defs>'''


class Node:
    def __init__(self, node_id: str, stencil: Stencil, data: Dict[str, Any], x: int = 0, y: int = 0):
        self.node_id = node_id
        self.stencil = stencil
        self.data = data
        self.x = x
        self.y = y
        self.width = stencil.width
        self.height = self._calculate_height()

    def _calculate_height(self) -> int:
        if hasattr(self.stencil, 'calculate_height'):
            return self.stencil.calculate_height(self.data)
        return self.stencil.height

    def _calculate_width(self) -> int:
        if hasattr(self.stencil, 'get_width'):
            return self.stencil.get_width(self.data)
        return self.stencil.width

    def render(self) -> str:
        return self.stencil.render(self.data, self.x, self.y)



from typing import List, Dict, Tuple
from ...core.models import LineType, Cardinality, Table
from ...core.layout import LayoutEngine
from .canvas import Canvas, Node
from .stencil import TableStencil
from .rendering import Edge


class SVGERDiagram:
    """SVG形式でER図を生成するクラス"""

    def __init__(
        self,
        default_line_type: LineType = LineType.STRAIGHT,
        min_width: int = 800,
        min_height: int = 600
    ):
        self.canvas = Canvas(min_width, min_height, default_line_type)
        self.nodes = self.canvas.nodes
        self.min_width = min_width
        self.min_height = min_height

    def add_table(self, table: Table, x: int = None, y: int = None) -> str:
        """
        テーブルを追加

        Args:
            table: Tableオブジェクト
            x, y: 座標（Noneの場合は自動配置）

        Returns:
            テーブルID（物理名）
        """
        table_id = table.name

        stencil = TableStencil()

        # Column dataclassをdict形式に変換
        columns_data = []
        for col in table.columns:
            columns_data.append({
                'name': col.name,
                'logical_name': col.logical_name,
                'type': col.type,
                'primary_key': col.primary_key,
                'nullable': col.nullable,
                'foreign_key': col.foreign_key,
                'index': col.index
            })

        data = {
            'table_name': table.name,
            'logical_name': table.logical_name,
            'columns': columns_data,
            'bgcolor': table.bgcolor,
            'use_gradient': table.use_gradient
        }

        # 幅を事前計算
        temp_node = Node(table_id, stencil, data, 0, 0)
        node_width = temp_node._calculate_width()

        auto_positioned = (x is None or y is None)
        if auto_positioned:
            x, y = self._get_next_position()
            # 幅チェックして改行判定
            if x + node_width > self.canvas.width - 50:
                x = 50
                y += self.canvas.row_height
                self.canvas.next_position = [x, y]

        node = Node(table_id, stencil, data, x, y)
        node.width = node_width
        self.canvas.add_node(node)

        # 次の配置位置を更新（自動配置の場合のみ）
        if auto_positioned:
            self._update_next_position(node_width)

        # キャンバスサイズを調整
        self._adjust_canvas_size_for_current_layout()

        return table_id

    def _get_next_position(self) -> Tuple[int, int]:
        """次の配置位置を取得"""
        x, y = self.canvas.next_position

        # 前回のノードの幅を考慮
        if self.nodes and x + 400 > self.canvas.width - 50:
            x = 50
            y += self.canvas.row_height

        self.canvas.next_position = [x, y]
        return x, y

    def _update_next_position(self, node_width: int):
        """次の配置位置を更新"""
        self.canvas.next_position[0] += node_width + 50

    def get_node(self, node_id: str) -> Node:
        """ノードを取得"""
        return self.canvas.get_node(node_id)

    def set_node_position(self, node_id: str, x: int, y: int):
        """ノードの位置を設定"""
        node = self.get_node(node_id)
        if node:
            node.x = x
            node.y = y

    def add_edge(self, from_node_id: str, to_node_id: str, line_type: LineType = None, cardinality: Cardinality = None):
        """
        エッジ（関係線）を追加

        Args:
            from_node_id: 開始ノードID
            to_node_id: 終了ノードID
            line_type: 線の種類
            cardinality: カーディナリティ
        """
        edge = Edge(from_node_id, to_node_id, line_type or self.canvas.default_line_type, cardinality)
        self.canvas.edges.append(edge)
        # エッジ追加後にレイアウトを最適化
        self._optimize_layout_for_edges()

    def _optimize_layout_for_edges(self):
        """Force-directedアルゴリズムでレイアウト最適化"""
        if not self.canvas.edges:
            # エッジがない場合でもキャンバスサイズを調整
            self._adjust_canvas_size_for_current_layout()
            return

        # Force-directedレイアウトを実行
        new_width, new_height = LayoutEngine.layout(
            self.nodes,
            self.canvas.edges,
            min_width=self.min_width,
            min_height=self.min_height
        )

        # キャンバスサイズを更新
        self._update_canvas_size(new_width, new_height)

    def _adjust_canvas_size_for_current_layout(self):
        """現在のレイアウトに基づいてキャンバスサイズを調整"""
        new_width, new_height = LayoutEngine.adjust_canvas_size(
            self.nodes,
            min_width=self.min_width,
            min_height=self.min_height
        )
        self._update_canvas_size(new_width, new_height)

    def _update_canvas_size(self, width: int, height: int):
        """キャンバスサイズを更新"""
        self.canvas.width = width
        self.canvas.height = height

    def render_svg(self) -> str:
        """
        SVG形式でレンダリング

        Returns:
            SVG XML文字列
        """
        node_parts = []
        for node in self.nodes:
            node_parts.append(node.render())

        edge_parts = self.canvas.render_edges()

        svg_content = f'''<svg xmlns="http://www.w3.org/2000/svg" width="{self.canvas.width}" height="{self.canvas.height}">
    {self.canvas.render_arrow_marker()}
    {chr(10).join(node_parts)}
    {chr(10).join(edge_parts)}
</svg>'''
        return svg_content

    def save_svg(self, output):
        """
        SVGファイルを保存する

        Args:
            output: ファイルパス(str)またはファイルライクオブジェクト
        """
        svg_content = self.render_svg()

        if isinstance(output, str):
            # ファイルパスの場合
            with open(output, 'w', encoding='utf-8') as f:
                f.write(svg_content)
        else:
            # ストリームオブジェクトの場合
            output.write(svg_content)



from abc import ABC, abstractmethod
from typing import List, Dict, Any
from ...core.text_metrics import calculate_text_width


class Stencil(ABC):
    def __init__(self, width: int = 200, height: int = 100):
        self.width = width
        self.height = height

    @abstractmethod
    def render(self, data: Dict[str, Any], x: int, y: int) -> str:
        pass


class TableStencil(Stencil):
    def __init__(self, width: int = 400, min_height: int = 100):
        super().__init__(width, min_height)
        self.min_height = min_height
        self.row_height = 22
        self.header_height = 25
        self.min_logical_width = 40
        self.min_marker_width = 6
        self.min_physical_width = 30
        self.min_data_type_width = 45
        self.min_constraint_width = 25
        self.padding = 10

    def _calculate_widths(self, data: Dict[str, Any]) -> Dict[str, int]:
        physical_name = data.get('table_name', 'Table')
        logical_name = data.get('logical_name', physical_name)
        columns = data.get('columns', [])

        # テーブル名の幅計算
        logical_table_width = max(self.min_logical_width, calculate_text_width(logical_name, True) + self.padding)
        physical_table_width = max(self.min_physical_width, calculate_text_width(physical_name, False) + self.padding)
        table_total_width = logical_table_width + physical_table_width

        # カラム名の幅計算
        max_logical_col_width = self.min_logical_width
        marker_width = self.min_marker_width
        max_physical_col_width = self.min_physical_width
        max_type_width = self.min_data_type_width
        max_constraint_width = self.min_constraint_width

        for column in columns:
            physical_col_name = column.get('name', '')
            logical_col_name = column.get('logical_name', physical_col_name)
            column_type = column.get('type', 'VARCHAR')

            # 制約文字列の計算
            constraints = []
            if column.get('foreign_key', False):
                constraints.append('FK')
            if column.get('index', False):
                constraints.append('IDX')
            constraint_text = ', '.join(constraints) if constraints else ''

            max_logical_col_width = max(max_logical_col_width, calculate_text_width(logical_col_name, True) + self.padding)
            max_physical_col_width = max(max_physical_col_width, calculate_text_width(physical_col_name, False) + self.padding)
            max_type_width = max(max_type_width, calculate_text_width(column_type, False) + self.padding)
            if constraint_text:
                max_constraint_width = max(max_constraint_width, calculate_text_width(constraint_text, False) + self.padding)

        # 制約欄に内容がない場合は幅0
        if max_constraint_width == self.min_constraint_width:
            has_constraints = any(column.get('foreign_key', False) or column.get('index', False) for column in columns)
            if not has_constraints:
                max_constraint_width = 0

        column_total_width = marker_width + max_logical_col_width + max_physical_col_width + max_type_width + max_constraint_width

        return {
            'table_logical_width': logical_table_width,
            'table_physical_width': physical_table_width,
            'table_total_width': table_total_width,
            'logical_width': max_logical_col_width,
            'marker_width': marker_width,
            'physical_width': max_physical_col_width,
            'type_width': max_type_width,
            'constraint_width': max_constraint_width,
            'column_total_width': column_total_width,
            'total_width': max(table_total_width, column_total_width)
        }

    def render(self, data: Dict[str, Any], x: int, y: int) -> str:
        physical_name = data.get('table_name', 'Table')
        logical_name = data.get('logical_name', physical_name)
        columns = data.get('columns', [])
        bgcolor = data.get('bgcolor', '#ffffff')
        use_gradient = data.get('use_gradient', False)

        pk_columns = [col for col in columns if col.get('primary_key', False)]
        regular_columns = [col for col in columns if not col.get('primary_key', False)]
        sorted_columns = pk_columns + regular_columns

        widths = self._calculate_widths(data)
        self.width = widths['total_width']

        pk_columns_count = len(pk_columns)
        regular_columns_count = len(regular_columns)
        extra_rows = 1 if pk_columns_count > 0 and regular_columns_count == 0 else 0
        actual_height = self.header_height + (len(columns) + extra_rows) * self.row_height

        svg_parts = []

        # グラデーション定義を追加（必要な場合）
        gradient_id = f'gradient_{id(data)}'
        if use_gradient:
            svg_parts.append(f'<defs>')
            svg_parts.append(f'  <linearGradient id="{gradient_id}" x1="0%" y1="0%" x2="100%" y2="0%">')
            svg_parts.append(f'    <stop offset="0%" style="stop-color:{bgcolor};stop-opacity:1" />')
            svg_parts.append(f'    <stop offset="100%" style="stop-color:#ffffff;stop-opacity:1" />')
            svg_parts.append(f'  </linearGradient>')
            svg_parts.append(f'</defs>')

        # テーブル全体の背景
        fill_value = f'url(#{gradient_id})' if use_gradient else bgcolor
        svg_parts.append(f'<rect x="{x}" y="{y}" width="{self.width}" height="{actual_height}" '
                        f'fill="{fill_value}" stroke="black" stroke-width="2"/>')

        # ヘッダーセクション
        svg_parts.append(f'<rect x="{x}" y="{y}" width="{self.width}" height="{self.header_height}" '
                        f'fill="{fill_value}" stroke="black" stroke-width="1"/>')

        # テーブル名表示
        table_display = f'{logical_name} ({physical_name})' if logical_name != physical_name else logical_name
        svg_parts.append(f'<text x="{x + 5}" y="{y + self.header_height//2 + 4}" '
                        f'font-family="Arial" font-size="12" font-weight="bold">'
                        f'{table_display}</text>')

        current_y = y + self.header_height
        pk_end_y = None

        for i, column in enumerate(sorted_columns):
            physical_col_name = column.get('name', f'Column{i}')
            logical_col_name = column.get('logical_name', physical_col_name)
            column_type = column.get('type', 'VARCHAR')
            is_primary = column.get('primary_key', False)
            is_nullable = column.get('nullable', True)
            is_foreign_key = column.get('foreign_key', False)

            text_color = "black"
            font_weight = "normal"

            constraints = []
            if is_foreign_key:
                constraints.append('FK')
            if column.get('index', False):
                constraints.append('IDX')
            constraint_text = ', '.join(constraints) if constraints else ''

            # セル位置計算
            marker_col_x = x
            logical_col_x = x + widths['marker_width']
            physical_col_x = x + widths['marker_width'] + widths['logical_width']
            type_col_x = x + widths['marker_width'] + widths['logical_width'] + widths['physical_width']
            constraint_col_x = x + widths['marker_width'] + widths['logical_width'] + widths['physical_width'] + widths['type_width']

            # カラムデータ表示
            svg_parts.append(f'<text x="{logical_col_x + 5}" y="{current_y + self.row_height//2 + 3}" '
                            f'font-family="Arial" font-size="10" font-weight="{font_weight}" fill="{text_color}">'
                            f'{logical_col_name}</text>')

            # NOT NULLマーカー表示
            if not is_nullable:
                marker_x = marker_col_x + 3
                marker_y = current_y + self.row_height // 2 - 3
                svg_parts.append(f'<rect x="{marker_x}" y="{marker_y}" '
                                f'width="3" height="6" fill="black"/>')

            svg_parts.append(f'<text x="{physical_col_x + 5}" y="{current_y + self.row_height//2 + 3}" '
                            f'font-family="Arial" font-size="10" fill="black">'
                            f'{physical_col_name}</text>')

            svg_parts.append(f'<text x="{type_col_x + 5}" y="{current_y + self.row_height//2 + 3}" '
                            f'font-family="Arial" font-size="9" fill="black">'
                            f'{column_type}</text>')

            # 制約欄
            if widths['constraint_width'] > 0:
                svg_parts.append(f'<text x="{constraint_col_x + 5}" y="{current_y + self.row_height//2 + 3}" '
                                f'font-family="Arial" font-size="9" fill="black">'
                                f'{constraint_text}</text>')

            current_y += self.row_height

            if is_primary:
                pk_end_y = current_y

        if pk_columns and pk_end_y:
            svg_parts.append(f'<line x1="{x}" y1="{pk_end_y}" x2="{x + widths["total_width"]}" y2="{pk_end_y}" '
                            f'stroke="black" stroke-width="1"/>')

        # PKのみのテーブルの場合、空行を追加
        if len(pk_columns) > 0 and len(regular_columns) == 0:
            current_y += self.row_height

        return '\n'.join(svg_parts)

    def calculate_height(self, data: Dict[str, Any]) -> int:
        columns = data.get('columns', [])
        pk_columns = [col for col in columns if col.get('primary_key', False)]
        regular_columns = [col for col in columns if not col.get('primary_key', False)]

        extra_rows = 1 if len(pk_columns) > 0 and len(regular_columns) == 0 else 0
        return self.header_height + (len(columns) + extra_rows) * self.row_height

    def get_width(self, data: Dict[str, Any]) -> int:
        widths = self._calculate_widths(data)
        return widths['total_width']



"""Force-directedレイアウトアルゴリズム

ノード間の斥力とエッジの引力をシミュレートして
自然で美しいレイアウトを生成する
"""
from typing import List, Dict, Tuple, Protocol, Set
from collections import defaultdict
import math


class LayoutNode(Protocol):
    """レイアウト計算用のノードプロトコル"""
    node_id: str
    x: int
    y: int
    width: int
    height: int


class LayoutEdge(Protocol):
    """レイアウト計算用のエッジプロトコル"""
    from_node_id: str
    to_node_id: str


class LayoutEngine:
    """
    Force-directedレイアウトエンジン

    接続されたノードを近くに配置し、
    重なりを解消して見やすいレイアウトを生成する
    """

    @staticmethod
    def layout(
        nodes: List[LayoutNode],
        edges: List[LayoutEdge],
        iterations: int = 200,
        margin: int = 50,
        min_width: int = 800,
        min_height: int = 600
    ) -> Tuple[int, int]:
        """
        Force-directedアルゴリズムでノードを配置

        Args:
            nodes: レイアウト対象のノードリスト
            edges: エッジリスト
            iterations: シミュレーション反復回数
            margin: キャンバス端のマージン
            min_width: キャンバスの最小幅（デフォルト: 800）
            min_height: キャンバスの最小高さ（デフォルト: 600）

        Returns:
            (canvas_width, canvas_height)
        """
        if not nodes:
            return min_width, min_height

        n = len(nodes)
        node_map = {node.node_id: node for node in nodes}

        # 接続情報を構築
        neighbors = defaultdict(set)
        degree = defaultdict(int)
        for edge in edges:
            if edge.from_node_id in node_map and edge.to_node_id in node_map:
                neighbors[edge.from_node_id].add(edge.to_node_id)
                neighbors[edge.to_node_id].add(edge.from_node_id)
                degree[edge.from_node_id] += 1
                degree[edge.to_node_id] += 1

        # 接続されたノードと孤立ノードを分離
        connected_nodes = [node for node in nodes if degree.get(node.node_id, 0) > 0]
        isolated_nodes = [node for node in nodes if degree.get(node.node_id, 0) == 0]

        # ノードサイズの平均
        avg_width = sum(node.width for node in nodes) / n
        avg_height = sum(node.height for node in nodes) / n

        # 理想的なエッジ長（接続ノード間の距離）
        ideal_length = max(avg_width, avg_height) * 1.8

        # 接続されたノードのみで初期配置とシミュレーション
        if connected_nodes:
            connected_node_map = {node.node_id: node for node in connected_nodes}

            # 初期配置: 接続の多いノードを中心に配置（接続ノードのみ）
            positions = LayoutEngine._initial_placement(
                connected_nodes, neighbors, degree, ideal_length, margin
            )

            # Force-directed simulation（接続ノードのみ）
            positions = LayoutEngine._force_directed_simulation(
                positions, connected_node_map, edges, neighbors, ideal_length, iterations
            )

            # 重なり解消（接続ノードのみ）
            positions = LayoutEngine._resolve_overlaps(positions, connected_node_map, 30)

            # 孤立ノードをシミュレーション後に配置
            if isolated_nodes:
                positions = LayoutEngine._place_isolated_nodes(
                    positions, isolated_nodes, ideal_length
                )
        else:
            # 全ノードが孤立している場合はグリッド配置
            positions = LayoutEngine._grid_placement(nodes, ideal_length, margin)

        # 重なり解消
        positions = LayoutEngine._resolve_overlaps(positions, node_map, 30)

        # 座標を正規化（左上をmarginに）
        min_x = min(pos[0] - node_map[nid].width / 2 for nid, pos in positions.items())
        min_y = min(pos[1] - node_map[nid].height / 2 for nid, pos in positions.items())

        max_width = 0
        max_height = 0

        for node in nodes:
            cx, cy = positions[node.node_id]
            node.x = int(cx - node.width / 2 - min_x + margin)
            node.y = int(cy - node.height / 2 - min_y + margin)

            max_width = max(max_width, node.x + node.width)
            max_height = max(max_height, node.y + node.height)

        # 計算されたサイズと最低サイズの大きい方を採用
        calculated_width = max_width + margin
        calculated_height = max_height + margin

        return max(calculated_width, min_width), max(calculated_height, min_height)

    @staticmethod
    def _initial_placement(
        nodes: List[LayoutNode],
        neighbors: Dict[str, Set[str]],
        degree: Dict[str, int],
        ideal_length: float,
        margin: int
    ) -> Dict[str, List[float]]:
        """接続構造に基づく初期配置"""
        positions: Dict[str, List[float]] = {}

        # 接続されたノードと孤立ノードを分離
        connected_nodes = [n for n in nodes if degree.get(n.node_id, 0) > 0]
        isolated_nodes = [n for n in nodes if degree.get(n.node_id, 0) == 0]

        # 接続されたノードがない場合は、全ノードをグリッド配置
        if not connected_nodes:
            return LayoutEngine._grid_placement(nodes, ideal_length, margin)

        # 接続されたノードを次数でソート（多いものから配置）
        sorted_connected = sorted(connected_nodes, key=lambda x: -degree.get(x.node_id, 0))
        n_connected = len(sorted_connected)

        # 最初のノード（最も接続の多いノード）を中心に
        center = ideal_length * math.sqrt(n_connected) / 2 + margin

        placed: Set[str] = set()
        for i, node in enumerate(sorted_connected):
            if i == 0:
                # 中心に配置
                positions[node.node_id] = [center, center]
                placed.add(node.node_id)
            else:
                # 既に配置された隣接ノードの近くに配置
                neighbor_positions = [
                    positions[nid] for nid in neighbors[node.node_id]
                    if nid in placed
                ]

                if neighbor_positions:
                    # 隣接ノードの重心を計算
                    avg_x = sum(p[0] for p in neighbor_positions) / len(neighbor_positions)
                    avg_y = sum(p[1] for p in neighbor_positions) / len(neighbor_positions)

                    # 重心から少しずらした位置に配置
                    angle = 2 * math.pi * i / n_connected
                    positions[node.node_id] = [
                        avg_x + ideal_length * 0.8 * math.cos(angle),
                        avg_y + ideal_length * 0.8 * math.sin(angle)
                    ]
                else:
                    # 隣接ノードが未配置なら、中心の周りに配置
                    angle = 2 * math.pi * i / n_connected
                    radius = ideal_length * (1 + i / n_connected)
                    positions[node.node_id] = [
                        center + radius * math.cos(angle),
                        center + radius * math.sin(angle)
                    ]
                placed.add(node.node_id)

        # 孤立ノードを接続グループの端に配置
        if isolated_nodes:
            positions = LayoutEngine._place_isolated_nodes(
                positions, isolated_nodes, ideal_length
            )

        return positions

    @staticmethod
    def _grid_placement(
        nodes: List[LayoutNode],
        ideal_length: float,
        margin: int
    ) -> Dict[str, List[float]]:
        """全ノードが孤立している場合のグリッド配置"""
        positions: Dict[str, List[float]] = {}
        n = len(nodes)
        cols = max(1, int(math.sqrt(n)))

        for i, node in enumerate(nodes):
            row = i // cols
            col = i % cols
            positions[node.node_id] = [
                margin + col * ideal_length * 1.5,
                margin + row * ideal_length * 1.5
            ]

        return positions

    @staticmethod
    def _place_isolated_nodes(
        positions: Dict[str, List[float]],
        isolated_nodes: List[LayoutNode],
        ideal_length: float
    ) -> Dict[str, List[float]]:
        """孤立ノードを接続グループの端に配置"""
        if not positions:
            return positions

        # 接続グループの境界を計算
        min_x = min(p[0] for p in positions.values())
        max_x = max(p[0] for p in positions.values())
        min_y = min(p[1] for p in positions.values())
        max_y = max(p[1] for p in positions.values())

        # 孤立ノードをグループの右側に縦に並べる
        start_x = max_x + ideal_length * 1.2
        start_y = min_y

        for i, node in enumerate(isolated_nodes):
            positions[node.node_id] = [
                start_x,
                start_y + i * ideal_length * 0.8
            ]

        return positions

    @staticmethod
    def _force_directed_simulation(
        positions: Dict[str, List[float]],
        node_map: Dict[str, LayoutNode],
        edges: List[LayoutEdge],
        neighbors: Dict[str, Set[str]],
        ideal_length: float,
        iterations: int
    ) -> Dict[str, List[float]]:
        """Force-directedシミュレーション"""
        positions = {k: list(v) for k, v in positions.items()}
        n = len(positions)

        if n <= 1:
            return positions

        # パラメータ
        k = ideal_length  # 理想距離
        temperature = k * 2  # 初期温度
        min_temp = 1.0

        for _ in range(iterations):
            forces: Dict[str, List[float]] = {nid: [0.0, 0.0] for nid in positions}

            # 斥力（全ノード間）
            node_ids = list(positions.keys())
            for i, nid1 in enumerate(node_ids):
                for nid2 in node_ids[i + 1:]:
                    dx = positions[nid1][0] - positions[nid2][0]
                    dy = positions[nid1][1] - positions[nid2][1]
                    dist_sq = dx * dx + dy * dy
                    dist = math.sqrt(dist_sq) if dist_sq > 0 else 0.1

                    # 斥力: k^2 / dist（遠いノードには弱い斥力）
                    repulsion = (k * k) / dist * 0.5

                    fx = (dx / dist) * repulsion
                    fy = (dy / dist) * repulsion

                    forces[nid1][0] += fx
                    forces[nid1][1] += fy
                    forces[nid2][0] -= fx
                    forces[nid2][1] -= fy

            # 引力（接続ノード間）- より強い引力
            for edge in edges:
                nid1, nid2 = edge.from_node_id, edge.to_node_id
                if nid1 not in positions or nid2 not in positions:
                    continue

                dx = positions[nid2][0] - positions[nid1][0]
                dy = positions[nid2][1] - positions[nid1][1]
                dist = math.sqrt(dx * dx + dy * dy)

                if dist < 0.1:
                    continue

                # 引力: dist^2 / k（強い引力で接続ノードを近づける）
                attraction = (dist * dist) / k * 1.5

                fx = (dx / dist) * attraction
                fy = (dy / dist) * attraction

                forces[nid1][0] += fx
                forces[nid1][1] += fy
                forces[nid2][0] -= fx
                forces[nid2][1] -= fy

            # 位置更新
            for nid in positions:
                fx, fy = forces[nid]
                force_mag = math.sqrt(fx * fx + fy * fy)
                if force_mag > 0.1:
                    # 温度で移動量を制限
                    scale = min(force_mag, temperature) / force_mag
                    positions[nid][0] += fx * scale
                    positions[nid][1] += fy * scale

            # 冷却
            temperature = max(temperature * 0.95, min_temp)

        return positions

    @staticmethod
    def _resolve_overlaps(
        positions: Dict[str, List[float]],
        node_map: Dict[str, LayoutNode],
        min_gap: int,
        iterations: int = 100
    ) -> Dict[str, List[float]]:
        """ノードの重なりを解消"""
        positions = {k: list(v) for k, v in positions.items()}

        for _ in range(iterations):
            moved = False
            node_ids = list(positions.keys())

            for i, nid1 in enumerate(node_ids):
                for nid2 in node_ids[i + 1:]:
                    n1 = node_map[nid1]
                    n2 = node_map[nid2]

                    x1, y1 = positions[nid1]
                    x2, y2 = positions[nid2]

                    # 必要な最小距離
                    min_dx = (n1.width + n2.width) / 2 + min_gap
                    min_dy = (n1.height + n2.height) / 2 + min_gap

                    dx = abs(x2 - x1)
                    dy = abs(y2 - y1)

                    # 重なり判定（矩形の重なり）
                    if dx < min_dx and dy < min_dy:
                        moved = True
                        overlap_x = min_dx - dx
                        overlap_y = min_dy - dy

                        # 最小の移動で解消
                        if overlap_x < overlap_y:
                            push = overlap_x / 2 + 1
                            if x1 < x2:
                                positions[nid1][0] -= push
                                positions[nid2][0] += push
                            else:
                                positions[nid1][0] += push
                                positions[nid2][0] -= push
                        else:
                            push = overlap_y / 2 + 1
                            if y1 < y2:
                                positions[nid1][1] -= push
                                positions[nid2][1] += push
                            else:
                                positions[nid1][1] += push
                                positions[nid2][1] -= push

            if not moved:
                break

        return positions

    @staticmethod
    def adjust_canvas_size(
        nodes: List[LayoutNode],
        margin: int = 50,
        min_width: int = 800,
        min_height: int = 600
    ) -> Tuple[int, int]:
        """現在のレイアウトに基づいてキャンバスサイズを計算"""
        if not nodes:
            return min_width, min_height

        max_width = 0
        max_height = 0

        for node in nodes:
            max_width = max(max_width, node.x + node.width)
            max_height = max(max_height, node.y + node.height)

        # 計算されたサイズと最低サイズの大きい方を採用
        calculated_width = max_width + margin
        calculated_height = max_height + margin

        return max(calculated_width, min_width), max(calculated_height, min_height)



from typing import List, Dict, Any, Tuple
from ...core.text_metrics import calculate_text_width
from .generator import DrawioGenerator


class DrawioTableStencil:
    """draw.io用のテーブルステンシル（テーブル形式 - SVGと同一レイアウト）"""

    def __init__(self, width: int = 400, min_height: int = 100):
        self.width = width
        self.min_height = min_height
        self.row_height = 22
        self.header_height = 25
        self.min_logical_width = 40
        self.min_marker_width = 6
        self.min_physical_width = 30
        self.min_data_type_width = 45
        self.min_constraint_width = 25
        self.padding = 10

    def _calculate_widths(self, data: Dict[str, Any]) -> Dict[str, int]:
        """テーブルとカラムの幅を計算"""
        physical_name = data.get('table_name', 'Table')
        logical_name = data.get('logical_name', physical_name)
        columns = data.get('columns', [])

        # テーブル名の幅計算
        logical_table_width = max(self.min_logical_width, calculate_text_width(logical_name, True) + self.padding)
        physical_table_width = max(self.min_physical_width, calculate_text_width(physical_name, False) + self.padding)
        table_total_width = logical_table_width + physical_table_width

        # カラム名の幅計算
        max_logical_col_width = self.min_logical_width
        marker_width = self.min_marker_width
        max_physical_col_width = self.min_physical_width
        max_type_width = self.min_data_type_width
        max_constraint_width = self.min_constraint_width

        for column in columns:
            physical_col_name = column.get('name', '')
            logical_col_name = column.get('logical_name', physical_col_name)
            column_type = column.get('type', 'VARCHAR')

            # 制約文字列の計算
            constraints = []
            if column.get('foreign_key', False):
                constraints.append('FK')
            if column.get('index', False):
                constraints.append('IDX')
            constraint_text = ', '.join(constraints) if constraints else ''

            max_logical_col_width = max(max_logical_col_width, calculate_text_width(logical_col_name, True) + self.padding)
            max_physical_col_width = max(max_physical_col_width, calculate_text_width(physical_col_name, False) + self.padding)
            max_type_width = max(max_type_width, calculate_text_width(column_type, False) + self.padding)
            if constraint_text:
                max_constraint_width = max(max_constraint_width, calculate_text_width(constraint_text, False) + self.padding)

        # 制約欄に内容がない場合は幅0
        if max_constraint_width == self.min_constraint_width:
            has_constraints = any(column.get('foreign_key', False) or column.get('index', False) for column in columns)
            if not has_constraints:
                max_constraint_width = 0

        column_total_width = marker_width + max_logical_col_width + max_physical_col_width + max_type_width + max_constraint_width

        return {
            'table_logical_width': logical_table_width,
            'table_physical_width': physical_table_width,
            'table_total_width': table_total_width,
            'logical_width': max_logical_col_width,
            'marker_width': marker_width,
            'physical_width': max_physical_col_width,
            'type_width': max_type_width,
            'constraint_width': max_constraint_width,
            'column_total_width': column_total_width,
            'total_width': max(table_total_width, column_total_width)
        }

    def calculate_height(self, data: Dict[str, Any]) -> int:
        """テーブルの高さを計算"""
        columns = data.get('columns', [])
        pk_columns = [col for col in columns if col.get('primary_key', False)]
        regular_columns = [col for col in columns if not col.get('primary_key', False)]

        # PKのみのテーブルの場合、空行を1行追加
        extra_rows = 1 if len(pk_columns) > 0 and len(regular_columns) == 0 else 0
        return self.header_height + (len(columns) + extra_rows) * self.row_height

    def get_width(self, data: Dict[str, Any]) -> int:
        """テーブルの幅を取得"""
        widths = self._calculate_widths(data)
        return widths['total_width']

    def render_mxcells(self, data: Dict[str, Any], x: int, y: int, canvas) -> Tuple[List[Dict[str, Any]], str]:
        """
        draw.io用のmxCellリストを生成（SVGと同一レイアウト、グループ化対応）

        Args:
            data: テーブルデータ
            x, y: 位置
            canvas: DrawioCanvasインスタンス（セルID生成用）

        Returns:
            (mxCellデータのリスト, グループセルID)
        """
        cells = []
        physical_name = data.get('table_name', 'Table')
        logical_name = data.get('logical_name', physical_name)
        columns = data.get('columns', [])
        bgcolor = data.get('bgcolor', '#ffffff')
        use_gradient = data.get('use_gradient', False)

        # PKカラムと通常カラムを分離
        pk_columns = [col for col in columns if col.get('primary_key', False)]
        regular_columns = [col for col in columns if not col.get('primary_key', False)]
        sorted_columns = pk_columns + regular_columns

        # 幅・高さ計算
        widths = self._calculate_widths(data)
        width = widths['total_width']
        height = self.calculate_height(data)

        # テーブル名表示（論理名 (物理名) 形式）
        table_display = f'{logical_name} ({physical_name})' if logical_name != physical_name else logical_name

        # 0. グループセルを作成（全要素の親）
        group_id = canvas.get_next_cell_id()
        group_cell = DrawioGenerator.create_group_cell(
            cell_id=group_id,
            x=x,
            y=y,
            width=width,
            height=height
        )
        cells.append(group_cell)

        # 以降の要素はグループ内の相対座標を使用
        # 1. テーブル外枠の矩形を作成
        table_rect_id = canvas.get_next_cell_id()
        table_rect = DrawioGenerator.create_table_rect(
            cell_id=table_rect_id,
            x=0,
            y=0,
            width=width,
            height=height,
            bgcolor=bgcolor,
            use_gradient=use_gradient,
            parent=group_id
        )
        cells.append(table_rect)

        # 2. ヘッダー背景の矩形を作成
        header_rect_id = canvas.get_next_cell_id()
        header_rect = DrawioGenerator.create_header_rect(
            cell_id=header_rect_id,
            x=0,
            y=0,
            width=width,
            height=self.header_height,
            bgcolor=bgcolor,
            use_gradient=use_gradient,
            parent=group_id
        )
        cells.append(header_rect)

        # 3. テーブル名テキストを作成
        header_text_id = canvas.get_next_cell_id()
        header_text = DrawioGenerator.create_text_cell(
            cell_id=header_text_id,
            value=table_display,
            x=5,
            y=0,
            width=width - 10,
            height=self.header_height,
            font_size=12,
            font_weight='bold',
            align='left',
            parent=group_id
        )
        cells.append(header_text)

        # 4. カラム行を作成
        current_y = self.header_height
        pk_end_y = None

        for i, column in enumerate(sorted_columns):
            physical_col_name = column.get('name', f'Column{i}')
            logical_col_name = column.get('logical_name', physical_col_name)
            column_type = column.get('type', 'VARCHAR')
            is_primary = column.get('primary_key', False)
            is_nullable = column.get('nullable', True)
            is_foreign_key = column.get('foreign_key', False)

            # 制約文字列
            constraints = []
            if is_foreign_key:
                constraints.append('FK')
            if column.get('index', False):
                constraints.append('IDX')
            constraint_text = ', '.join(constraints) if constraints else ''

            # セル位置計算（グループ内の相対座標）
            marker_col_x = 0
            logical_col_x = widths['marker_width']
            physical_col_x = widths['marker_width'] + widths['logical_width']
            type_col_x = widths['marker_width'] + widths['logical_width'] + widths['physical_width']
            constraint_col_x = widths['marker_width'] + widths['logical_width'] + widths['physical_width'] + widths['type_width']

            # NOT NULLマーカー表示
            if not is_nullable:
                marker_id = canvas.get_next_cell_id()
                marker_x = marker_col_x + 3
                marker_y = current_y + self.row_height // 2 - 3
                marker_cell = DrawioGenerator.create_marker_rect(
                    cell_id=marker_id,
                    x=marker_x,
                    y=marker_y,
                    width=3,
                    height=6,
                    parent=group_id
                )
                cells.append(marker_cell)

            # 論理名テキスト
            logical_text_id = canvas.get_next_cell_id()
            logical_text = DrawioGenerator.create_text_cell(
                cell_id=logical_text_id,
                value=logical_col_name,
                x=logical_col_x + 5,
                y=current_y,
                width=widths['logical_width'] - 5,
                height=self.row_height,
                font_size=10,
                font_weight='normal',
                align='left',
                parent=group_id
            )
            cells.append(logical_text)

            # 物理名テキスト
            physical_text_id = canvas.get_next_cell_id()
            physical_text = DrawioGenerator.create_text_cell(
                cell_id=physical_text_id,
                value=physical_col_name,
                x=physical_col_x + 5,
                y=current_y,
                width=widths['physical_width'] - 5,
                height=self.row_height,
                font_size=10,
                font_weight='normal',
                align='left',
                parent=group_id
            )
            cells.append(physical_text)

            # データ型テキスト
            type_text_id = canvas.get_next_cell_id()
            type_text = DrawioGenerator.create_text_cell(
                cell_id=type_text_id,
                value=column_type,
                x=type_col_x + 5,
                y=current_y,
                width=widths['type_width'] - 5,
                height=self.row_height,
                font_size=9,
                font_weight='normal',
                align='left',
                parent=group_id
            )
            cells.append(type_text)

            # 制約テキスト（制約欄がある場合のみ）
            if widths['constraint_width'] > 0:
                constraint_text_id = canvas.get_next_cell_id()
                constraint_cell = DrawioGenerator.create_text_cell(
                    cell_id=constraint_text_id,
                    value=constraint_text,
                    x=constraint_col_x + 5,
                    y=current_y,
                    width=widths['constraint_width'] - 5,
                    height=self.row_height,
                    font_size=9,
                    font_weight='normal',
                    align='left',
                    parent=group_id
                )
                cells.append(constraint_cell)

            current_y += self.row_height

            if is_primary:
                pk_end_y = current_y

        # 5. PK区切り線を作成（PKカラムがある場合）- テーブル幅全体に
        if pk_columns and pk_end_y and regular_columns:
            pk_line_id = canvas.get_next_cell_id()
            pk_line = DrawioGenerator.create_horizontal_line(
                cell_id=pk_line_id,
                x=0,
                y=pk_end_y,
                width=width,  # total_widthを使用
                stroke_width=1,
                parent=group_id
            )
            cells.append(pk_line)

        # グループセルIDを返す（エッジ接続用）
        return cells, group_id
